<?php	
	/*
	 * Cette fonction permet d'ajouter une case dans la BDD
	 */
	function ajouterCasePlanParking ($bdd, $tab){
		$sql = 'INSERT INTO CasePlanParking(idParking, numeroLigne, numeroColonne, idType, idPlace) '.
			' VALUES(:idParking, :numeroLigne, :numeroColonne, :idType, :idPlace)';
		$req = $bdd->prepare($sql);
		$req->execute(array(
				'idParking' => intval($tab['idParking']),
				'numeroColonne' => intval($tab['numeroColonne']),
				'numeroLigne' => intval($tab['numeroLigne']),
				'idType' => intval($tab['idType']),
				'idPlace' => intval($tab['idPlace'])));
		$req->closeCursor();
	}
	
	/*
	 * Cette fonction permet de changer les informations de la casePlanParking dont les lignes et colonnes
	 * sont placés dans le tableau placé en paramètres
	 */
	function modifierCasePlanParking ($bdd, $tab)
	{
		foreach ($tab as $champ => $valeur){
			$sql = 'UPDATE CasePlanParking SET ' . strval($champ) . ' = :valeur WHERE numeroColonne= :numeroColonne ' . 
				' AND numeroLigne= :numeroLigne AND idParking= :id;';
			$req = $bdd->prepare($sql);
			$req->execute(array(
					'valeur' => $valeur,
					'numeroColonne' => $tab['numeroColonne'],
					'numeroLigne' => $tab['numeroLigne'],
					'id' => $tab['idParking']));
			$req->closeCursor();
		}
	}
	
	/*
	 * Cette fonction supprimer une CasePlanParking
	 */
	function supprimerCasePlanParking ($bdd, $tab){
		$sql = 'DELETE FROM CasePlanParking WHERE idParking= :idParking ';
		$req = $bdd->prepare($sql);
		$req->execute(array('idParking' => $tab['idParking']));
		$req->closeCursor();
	}
	
	/*
	 * Cette fonction renvoie true si la case existe déjà, sinon renvoie false
	 */
	 function caseDejaExistante ($bdd, $id, $numeroColonne, $numeroLigne){
		$sql = 'SELECT idParking FROM CasePlanParking WHERE idParking= :id AND numeroColonne= :numeroColonne AND numeroLigne= :numeroLigne';
		$req = $bdd->prepare ($sql);
		$req->execute(array(
				'id' => $id,
				'numeroColonne' => $numeroColonne,
				'numeroLigne' => $numeroLigne));
		$resultat = $req->fetch();
		if(!$resultat)
		{
			$existe = false;
		}
		else 
		{
			$existe = true;
		}
		$req->closeCursor();
		return $existe;
	 }
	 
	 /*
	  * Cette fonction permet de récupérer l'id de la place associée à la case 
	  */
	  function recupererIdPlaceParCase ($bdd, $idParking, $numColonne, $numLigne){
		$sql = 'SELECT idPlace FROM CasePlanParking WHERE numeroLigne= :numeroLigne AND numeroColonne= :numeroColonne AND idParking = :idParking';
		$req = $bdd->prepare($sql);
		$req->execute(array(
				'numeroLigne' => $numLigne,
				'idParking' => $idParking,
				'numeroColonne' => $numColonne));
		if($ligne = $req->fetch())
		{
			$idPlace = $ligne['idPlace'];
		}
		$req->closeCursor();
		return $idPlace;
	}
	
	/*
	 * Cette fonction permet de recuperer toutes les cases d'un parking
	 */
	 function recupererCasesParking ($bdd, $idParking){
		$sql = 'SELECT * FROM CasePlanParking WHERE idParking = :idParking';
		$req = $bdd->prepare($sql);
		$req->execute(array('idParking' => $idParking));
		$i = 0;
		while ($ligne = $req->fetch(PDO::FETCH_ASSOC))
		{
			$tab[$i] = $ligne;
			$i++;
		}
		$req->closeCursor();
		return $tab;
		
	 }
	 
	/*
	 * Cette fonction permet de renvoyer toutes les places renseignées dans la table case
	 */
	function recupPlacesCases ($bdd, $idParking){
		$sql = 'SELECT idPlace FROM CasePlanParking WHERE idParking = :idParking';
		$req = $bdd->prepare($sql);
		$req->execute(array('idParking' => $idParking));
		$i = 0;
		while ($ligne = $req->fetch())
		{
			$tab[$i] = $ligne ['idPlace'];
			$i++;
		}
		$req->closeCursor();
		return $tab;
	}
?>