<?php
	/*
	 * extraire le pseudo de la BDD.
	 */
	function extraireIDParMail($bdd, $mail)
	{
		$sql = 'SELECT pseudo FROM membres WHERE adresse_mail = :mail';
		$req = $bdd->prepare ($sql);
		$req->execute(array('mail' => $mail));
		if($ligne = $req->fetch())
		{
			$pseudo = $ligne['pseudo'];
		}
		$req->closeCursor();
		return $pseudo;
	}
	
	/*
	 * extraire les identifiants (login et mot de passe) de la BDD.
	 */
	function extraireIDAndPass($bdd, $pseudo)
	{
		$tab = array();
		$sql = 'SELECT pseudo, mdp FROM membres WHERE pseudo = :pseudo';
		$req = $bdd->prepare ($sql);
		$req->execute(array('pseudo' => $pseudo));
		if($ligne = $req->fetch())
		{
			$tab['pseudo']=$ligne['pseudo'];
			$tab['mdp']=$ligne['mdp'];
		}
		$req->closeCursor();
		return $tab;
	}
	
	/*
	 * vérifier si l'identifiant existe déjà dans la bdd
	 */
	function iDDejaExistant ($bdd, $pseudo)
	{
		$sql = 'SELECT pseudo FROM membres WHERE pseudo= :pseudo';
		$req = $bdd->prepare ($sql);
		$req->execute(array('pseudo' => $pseudo));
		$resultat = $req->fetch();
		if(!$resultat)
		{
			$existe = false;
		}
		else 
		{
			$existe = true;
		}
		$req->closeCursor();
		return $existe;
	}
	
	/*
	 * vérifier si l'adresse mail existe déjà dans la bdd
	 */
	function mailDejaExistant ($bdd, $mail)
	{
		$sql = 'SELECT adresse_mail FROM membres WHERE LOWER(adresse_mail)= :mail';
		$req = $bdd->prepare ($sql);
		$req->execute(array('mail' => $mail));
		$resultat = $req->fetch();
		if(!$resultat)
		{
			$existe = false;
		}
		else 
		{
			$existe = true;
		}
		$req->closeCursor();
		return $existe;
	}
	
	/*
	 * Cette fonction permet d'ajouter un membre dans la bdd en étant d'origine
     * avec la fonction "membre"
	 */
	function ajouterMembre ($bdd, $tab){
		if(array_key_exists("id_groupe", $tab))
		{
			$id_groupe = $tab['id_groupe'];
		}
		else 
		{
			$id_groupe = 4;
		}
		$sql = 'INSERT INTO membres(id, pseudo, mdp, adresse_mail, date_inscription, id_groupe) '.
			' VALUES(0, :pseudo, :pass, :email, CURDATE(), :idGroupe)';
		$req = $bdd->prepare($sql);
		$req->execute(array(
				'pseudo' => strval($tab['pseudo']),
				'pass' => strval($tab['mdp']),
				'email' => strval($tab['mail']),
				'idGroupe' => intval($id_groupe)));
		$req->closeCursor();
	}
	
	/*
	 * Cette fonction permet de changer le mot de passe d'un membre 
	 * (recherche du membre grâce à son pseudo) dans la bdd
	 */
	function modifierMembre ($bdd, $pseudo, $champ, $valeur)
	{
		$sql = 'UPDATE membres SET ' . strval($champ) . ' = :valeur WHERE pseudo= :pseudo';
		$req = $bdd->prepare($sql);
		$req->execute(array(
				'pseudo' => strval($pseudo),
				'valeur' => strval($valeur)));
		$req->closeCursor();
	}
	
	/*
	 * Cette fonction supprimer un membre dans la bdd
	 */
	function supprimerMembre ($bdd, $pseudo){
		$sql = 'DELETE FROM membres WHERE pseudo= :pseudo';
		$req = $bdd->prepare($sql);
		$req->execute(array('pseudo' => $pseudo));
		$req->closeCursor();
	}	


	/*
	 * Cette fonction permet de renvoyer une variable etat 
	 * Sinon etat est nulle alors on peut ajouter un membre, sinon non
	 */
	function verificationAjoutMembre ($bdd, &$etat)
	{
		$pass = htmlspecialchars($_POST['password']);
		$pseudo = htmlspecialchars($_POST['pseudo']); 
		$pseudo=str_replace(CHR(32), '', $pseudo); //enlève les espaces 
		$conf_pass = htmlspecialchars($_POST['conf_password']);
		$mail = strtolower(htmlspecialchars($_POST['mail']));
		
		// si les variables ne sont pas nulles
		if($pseudo != NULL && $pass != NULL && $mail!= null && $conf_pass != null) 
		{
			// On crypte le mot de passe et la confirmation
			$cryptPass = crypterPassWrd($pass);
			$cryptConfPass = crypterPassWrd($conf_pass);
			
			$etat = 0;
			if (mailDejaExistant($bdd, $mail))
			{
				$etat += 1;
			}
			if (!bonFormatAdresseMail($mail))
			{
				$etat += 2;
			}
			if (iDDejaExistant($bdd, $pseudo))
			{
				$etat += 4;
			}
			if ($cryptPass != $cryptConfPass){
				$etat += 8;
			}
			if(strlen($pass) < 4){
				$etat += 16;
			}
		}
		else 
		{
			$etat = 32;
		}
		$tab = array(
			'pseudo' => $pseudo,
			'mail' => $mail,
			'mdp' => $cryptPass);
		return $tab;
	}
	
?>