<?php	
	/*
	 * extraire le nombre de places disponibles de la table parking
	 */
	function extraireNbPlacesDisposParNom($bdd, $id)
	{
		$sql = 'SELECT NbDispoPlaces FROM parking WHERE idParking = :id';
		$req = $bdd->prepare ($sql);
		$req->execute(array('id' => $id));
		if($ligne = $req->fetch())
		{
			$nbPlaces = $ligne['NbDispoPlaces'];
		}
		$req->closeCursor();
		return $nbPlaces;
	}
	
	/*
	 * extraire le nom (avec la rue et le code postal) des parkings grâce à un champ 
	 * et sa valeurs placés en paramètres
	 */
	function extraireParkings($bdd, $champ, $valeur)
	{
		$sql = 'SELECT idParking, Nom, Rue, CodePostal, Longitude, Latitude FROM parking WHERE ' . $champ . ' = ' . $valeur . ';';
		$req = $bdd->prepare ($sql);
		$req->execute();
		$i = 0;
		// recupération de toutes les colonnes
		while($row = $req->fetch(PDO::FETCH_ASSOC)) {
			// recupération de toutes les lignes
			$parking[$i] = $row;
			$i++;
		}
		$req->closeCursor();
		return $parking;
	}
	
	/*
	 * extraire le nom (avec la rue et le code postal) des parkings grâce au nom de la ville (placée en paramètre)
	 */
	function extraireInfosParID($bdd, $id)
	{
		$sql = 'SELECT parking.*, villes.nom_ville as Ville FROM parking INNER JOIN villes on parking.id_ville = villes.id WHERE idParking = ' . $id . ';';
		$req = $bdd->prepare ($sql);
		$req->execute();
		// recupération de toutes les colonnes
		$parking = $req->fetch(PDO::FETCH_ASSOC);
		$req->closeCursor();
		return $parking;
	}
	
	/*
	 * mettre a jour les valeurs du parking dont l'id est placé en paramètre
	 */
	function miseAJourParkingParID($bdd, $id, $champ, $valeur)
	{
		$sql = 'UPDATE parking SET ' . strval($champ) . ' = :valeur WHERE idParking= :idParking';
		$req = $bdd->prepare($sql);
		$req->execute(array(
				'idParking' => strval($id),
				'valeur' => strval($valeur)));
		$req->closeCursor();
	}
	
	
	/*
	 * recuperer les dimensions du parking
	 */
	function recupererDimensionParking($bdd, $id)
	{
		$sql = 'SELECT NbColonnes, NbLignes FROM parking WHERE idParking= ' . $id . ';';
		$req = $bdd->prepare ($sql);
		$req->execute();
		// recupération de toutes les colonnes
		if ($ligne = $req->fetch(PDO::FETCH_ASSOC)){
			$dimension['nbreColonnes'] = $ligne['NbColonnes'];
			$dimension['nbreLignes'] = $ligne['NbLignes'];
		}
		$req->closeCursor();
		return $dimension;
	}
	
	/*
	 * Modifier les paramètres du parking
	 */
	function modifierParking($bdd, $id, $tab)
	{
		foreach ($tab as $champ => $valeur){
			$sql = 'UPDATE parking SET ' . strval($champ) . ' = :valeur WHERE idParking= :idParking';
			$req = $bdd->prepare($sql);
			$req->execute(array(
					'idParking' => strval($id),
					'valeur' => strval($valeur)));
			$req->closeCursor();
		}
	}
	
	/*
	 * ajouter un parking
	 */
	function ajouterParking ($bdd, $tab){
		$sql = 'INSERT INTO place(idParking, Nom, NbMaxPlaces, NbDispoPlaces, JoursOuverts, Horaires, Tarif, Type, Avis, Rue, CodePostal, id_ville, InformationsComplementaires) '.
			' VALUES(Null, :nom, :nbMax, :nbDispo, :joursOuverts, :horaires, :tarif, :type, :avis, :rue, :codePostal, :id_ville, :infoCompl)';
		$req = $bdd->prepare($sql);
		$req->execute(array(
				'nom' => $tab['Nom'],
				'nbMax' => $tab['NbMaxPlaces'],
				'nbDispo' => $tab['NbDispoPlaces'],
				'joursOuverts' => $tab['JoursOuverts'],
				'horaires' => $tab['Horaires'],
				'tarif' => $tab['Tarif'],
				'type' => $tab['Type'],
				'avis' => $tab['Avis'],
				'rue' => $tab['Rue'],
				'codePostal' => $tab['CodePostal'],
				'id_ville' => $tab['id_ville'],
				'infoCompl' => $tab['InformationsComplementaires']));
		$req->closeCursor();
	}
	
	/*
	 * Cette fonction retourne true s'il existe déjà le parking, sinon false
	 */
	 function parkingExiste ($bdd, $nom){
		$sql = 'SELECT Nom FROM parking WHERE Nom= :nom';
		$req = $bdd->prepare ($sql);
		$req->execute(array('nom' => $nom));
		$resultat = $req->fetch();
		$req->closeCursor();
		if(!resultat)
		{
			return false;
		}
		else {
			return true;
		}
	 }

	/*
         * Cette fonction retourne true s'il existe déjà un id de parking, sinon false
         */
         function idParkingExiste ($bdd, $id){
                $sql = 'SELECT Nom FROM parking WHERE idParking= :idParking';
                $req = $bdd->prepare ($sql);
                $req->execute(array('idParking' => $idParking));
                $resultat = $req->fetch();
                $req->closeCursor();
                if(!resultat)
                {
                        return false;
                }
                else {
                        return true;
                }
         }


	/*
         * Cette fonction modifie le nombre de places dispos
         */
         function modifierNbPlacesDispos ($bdd, $idParking){
                $sql = 'SELECT count(*) as nbre FROM capteur WHERE idParking= :idParking AND disponible="1"';
                $req = $bdd->prepare ($sql);
                $req->execute(array('idParking' => $idParking));
                if($ligne = $req->fetch()){
			$resultat =$ligne['nbre'];
			miseAJourParkingParID($bdd, $idParking, "NbDispoPlaces", $resultat);
		} 
                $req->closeCursor();
         }

	/*
         * Cette fonction modifie le nombre de places maximum
         */
         function modifierNbPlacesMax ($bdd, $idParking){
                $sql = 'SELECT count(*) as nbre FROM capteur WHERE idParking= :idParking';
                $req = $bdd->prepare ($sql);
                $req->execute(array('idParking' => $idParking));
                if($ligne = $req->fetch()){
                        $resultat =$ligne['nbre'];
                        miseAJourParkingParID($bdd, $idParking, "NbMaxPlaces", $resultat);
                }
                $req->closeCursor();
         }


?>
