#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <poll.h>
#include <arpa/inet.h>


#include <stddef.h>
#include <fcntl.h>
#include <termios.h>
 
#include <openssl/rand.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

#define    MAX_BUFFER    1024

#define SERIALDEV	"/dev/ttyUSB1"
#define BAUDRATE	B9600 
#define BUFFER_SIZE	10


// Simple structure to keep track of the handle, and
// of what needs to be freed later.
typedef struct {
    int socket;
    SSL *sslHandle;
    SSL_CTX *sslContext;
} connection;


// Variable globale
static struct termios sauvegarde;

int ouvertureSerie(char *periph,int vitesse){
    struct termios nouveau;
    int df=open(periph,O_RDWR|O_NOCTTY);
    if(df<0) return -1;

    tcgetattr(df,&sauvegarde); /* save current port settings */
    bzero(&nouveau,sizeof(nouveau));
    nouveau.c_cflag=CLOCAL|CREAD|vitesse|CS8;
    nouveau.c_iflag=0;
    nouveau.c_oflag=0;
    nouveau.c_lflag=0;
    nouveau.c_cc[VTIME]=0;
    nouveau.c_cc[VMIN]=1;
    tcflush(df, TCIFLUSH);
    tcsetattr(df,TCSANOW,&nouveau);

    return df;
}

/** Fermeture d'un port serie **/
void fermetureSerie(int df){
    tcsetattr(df,TCSANOW,&sauvegarde);
    close(df);
}

int initialisationSerie(int ds){
	ds = ouvertureSerie(SERIALDEV,BAUDRATE);
	if(ds < 0){
		fprintf(stderr,"Erreur sur la connexion série.\n");
		exit(-1);
	}
	return ds;
}

void ecritureSerie(int ds){
	unsigned char cmd[] = "Init \r";
	unsigned char n_written = 0;
	unsigned char spot = 0;

	do{
		n_written = write(ds, &cmd[spot], 1);
		spot += n_written;
	}while(cmd[spot-1] != '\r' && n_written > 0);
}

void lectureSerie(int ds, char resp[BUFFER_SIZE]){
	unsigned char n = 0, spot = 0;
	unsigned char buf = '\0';
	
	do{
		n = read(ds, &buf, 1);
		sprintf(&resp[spot], "%c", buf);
		spot += n;
	}while(buf != '\0' && n > 0);

	if(n < 0)
		printf("Error reading.\n");
	else if(n == 0)
		printf("Read nothing!\n");
}



int connexionServeur(char *hote,char *service){
    struct addrinfo precisions,*resultat;
    int statut;
    int s;

    /* Creation de l'adresse de socket */
    memset(&precisions,0,sizeof precisions);
    precisions.ai_family=AF_UNSPEC;
    precisions.ai_socktype=SOCK_STREAM;
    statut=getaddrinfo(hote,service,&precisions,&resultat);
    if(statut<0){ perror("connexionServeur.getaddrinfo"); exit(EXIT_FAILURE); }

    /* Creation d'une socket */
    s=socket(resultat->ai_family,resultat->ai_socktype,resultat->ai_protocol);
    if(s<0){ perror("connexionServeur.socket"); exit(EXIT_FAILURE); }

    /* Connection de la socket a l'hote */
    if(connect(s,resultat->ai_addr,resultat->ai_addrlen)<0) return -1;

    /* Liberation de la structure d'informations */
    freeaddrinfo(resultat);

    return s;
}

// Establish a connection using an SSL layer
void sslConnect(connection *c)
{
  c->sslHandle = NULL;
  c->sslContext = NULL;
  if (c->socket)
    {
      // Register the available ciphers and digests
      SSL_library_init ();
      OpenSSL_add_all_algorithms();
      // Register the error strings for libcrypto & libssl
      SSL_load_error_strings ();

      // New context saying we are a client, and using SSL 2 or 3
      c->sslContext = SSL_CTX_new (SSLv23_client_method ());
      if (c->sslContext == NULL)
        { ERR_print_errors_fp (stderr); exit(EXIT_FAILURE); }

      // Define certs to use
      SSL_CTX_set_options(c->sslContext,SSL_OP_SINGLE_DH_USE);
      if(SSL_CTX_use_certificate_file(c->sslContext, "/etc/ssl/certs/ssl-cert-snakeoil.pem" , SSL_FILETYPE_PEM)<0)
        { ERR_print_errors_fp (stderr); exit(EXIT_FAILURE); }
      if(SSL_CTX_use_PrivateKey_file(c->sslContext, "/etc/ssl/private/ssl-cert-snakeoil.key", SSL_FILETYPE_PEM)<0)
        { ERR_print_errors_fp (stderr); exit(EXIT_FAILURE); }
        
      // Create an SSL struct for the connection
      c->sslHandle = SSL_new (c->sslContext);
      if (c->sslHandle == NULL)
        { ERR_print_errors_fp (stderr); exit(EXIT_FAILURE); }

      // Connect the SSL struct to our connection
      if (!SSL_set_fd (c->sslHandle, c->socket))
        { ERR_print_errors_fp (stderr); exit(EXIT_FAILURE); }

      // Initiate SSL handshake
      if (SSL_connect (c->sslHandle) != 1)
        { ERR_print_errors_fp (stderr); exit(EXIT_FAILURE); }      
    }
  else { perror ("Connect failed");  exit(EXIT_FAILURE); }
}

// Disconnect & free connection struct
void sslDisconnect (connection *c)
{
  if (c->socket)
    close (c->socket);
  if (c->sslHandle)
    {
      SSL_shutdown (c->sslHandle);
      SSL_free (c->sslHandle);
    }
  if (c->sslContext)
    SSL_CTX_free (c->sslContext);
}

// Read all available text from the connection
char *sslRead (connection *c)
{
	char *rc = NULL;
	int received, total = 0;
	char buffer[MAX_BUFFER];

	if (c)
		while (1)
		{
			received = SSL_read (c->sslHandle, buffer, MAX_BUFFER);
			rc = realloc(rc,total+received);
			memcpy(rc+total,buffer,received);
			total += received;
			if (received < MAX_BUFFER) break;
		}
		if(rc!=NULL){
			rc = realloc(rc,total+1);
			rc[total]='\0';
		} 
		return rc;
}

// Write text to the connection
void sslWrite (connection *c, char *text)
{
  if (c)
    SSL_write (c->sslHandle, text, strlen (text));
}

int longueurChaine (char* donnees){
	int i=0;
	while (donnees[i] != '\0'){
		i++;
	}
	return i;
}

int main(int argc,char *argv[])
{
	connection c;
    char *machine = "193.48.57.56";
    char *port = "40100";
    int idParking = 1;
    int idCapteur = 1;
    char etatCapteur = 0;
    char longueur[40];
    char donnees[100];
    int ds = 0;	// Descriptor serial
	// Whole response
	char resp[BUFFER_SIZE];
	
	while(1){
		char chaine[400] = "POST /AAP/php/miseAJourCapteurs.php HTTP/1.1\r\n\
Host: mathieu.gerier.plil.net\r\n\
Content-Type: application/x-www-form-urlencoded\r\n";


		memset(resp, '\0', sizeof(resp));

		ds = initialisationSerie(ds);
		printf("Ouverture du port serie.\n");

		// Fonction bloquante...
		lectureSerie(ds, resp);
		
		idParking = resp[1];
		idCapteur = resp[2];
		etatCapteur = resp[3];
		
printf("parking: %d\t capteur: %d\t etat: %d\n", idParking, idCapteur, etatCapteur);

		//fermeture série
		fermetureSerie(ds);
		printf("Fermeture du port serie.\n");

		sprintf(donnees, "Action=GetStatus&idParking=%u&idCapteur=%u&etatCapteur=%u", idParking, idCapteur, etatCapteur);
		sprintf(longueur, "Content-Length: %i\r\n\r\n", longueurChaine(donnees));
		strcat(chaine, longueur); 
		strcat(chaine, donnees);

printf("message: ");
printf("%s", chaine);

		// Connection au serveur 
		c.socket = connexionServeur(machine,port);
		if(c.socket<0){ 
			fprintf(stderr,"Erreur de connexion au serveur\n"); exit(EXIT_FAILURE); 
		}
		printf("Connexion TCP Réussie\n");
		sslConnect(&c);
		
		// Envoi du POST             
		sslWrite(&c,chaine);
		/*sleep(1);
		char *reponse;
		while((reponse=sslRead(&c))!=NULL){
		  printf("reponse ->\n%s\n",reponse);
		  free(reponse);
		  }*/

		// On termine la connexion 
		sslDisconnect(&c);
	}
    
    return 0;
}
